@@ -33,7 +33,7 @@ import jakarta.servlet.WriteListener;
 
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
-import org.apache.tomcat.util.buf.B2CConverter;
+import org.apache.tomcat.util.buf.CharsetHolder;
 import org.apache.tomcat.util.buf.MessageBytes;
 import org.apache.tomcat.util.http.MimeHeaders;
 import org.apache.tomcat.util.http.parser.MediaType;
@@ -41,6 +41,12 @@ import org.apache.tomcat.util.res.StringManager;
 
 /**
  * Response object.
+ *
+ * @author James Duncan Davidson [duncan@eng.sun.com]
+ * @author Jason Hunter [jch@eng.sun.com]
+ * @author James Todd [gonzo@eng.sun.com]
+ * @author Harish Prabandham
+ * @author Hans Bergsten [hans@gefionsoftware.com]
  */
 public final class Response {
 
@@ -107,11 +113,7 @@ public final class Response {
      */
     String contentType = null;
     String contentLanguage = null;
-    Charset charset = null;
-    // Retain the original name used to set the charset so exactly that name is
-    // used in the ContentType header. Some (arguably non-specification
-    // compliant) user agents are very particular
-    String characterEncoding = null;
+    private CharsetHolder charsetHolder = CharsetHolder.EMPTY;
     long contentLength = -1;
     private Locale locale = DEFAULT_LOCALE;
 
@@ -300,15 +302,10 @@ public final class Response {
 
 
     /**
-     * Set the error flag.
-     *
-     * @return <code>false</code> if the error flag was already set
-     *
-     * @deprecated This method will be changed to return void in Tomcat 11 onwards
+     * Set the error flag if not already set.
      */
-    @Deprecated
-    public boolean setError() {
-        return errorState.compareAndSet(0, 1);
+    public void setError() {
+        errorState.compareAndSet(0, 1);
     }
 
 
@@ -370,7 +367,11 @@ public final class Response {
                 return;
             }
         }
-        headers.setValue(name).setString(value);
+        if (value == null) {
+            headers.removeHeader(name);
+        } else {
+            headers.setValue(name).setString(value);
+        }
     }
 
 
@@ -423,6 +424,10 @@ public final class Response {
         }
         if (name.equalsIgnoreCase("Content-Length")) {
             try {
+                if (value == null) {
+                    setContentLength(-1);
+                    return true;
+                }
                 long cL = Long.parseLong(value);
                 setContentLength(cL);
                 return true;
@@ -436,17 +441,6 @@ public final class Response {
     }
 
 
-    /**
-     * Signal that we're done with the headers, and body will follow.
-     *
-     * @deprecated Unused. Will be removed in Tomcat 11 onwards. Use {@link #commit()}.
-     */
-    @Deprecated
-    public void sendHeaders() {
-        commit();
-    }
-
-
     /**
      * Signal that we're done with the headers, and body will follow.
      */
@@ -500,32 +494,53 @@ public final class Response {
      * @param characterEncoding The name of character encoding.
      *
      * @throws UnsupportedEncodingException If the specified name is not recognised
+     *
+     * @deprecated Unused. Will be removed in Tomcat 12.
      */
+    @Deprecated
     public void setCharacterEncoding(String characterEncoding) throws UnsupportedEncodingException {
         if (isCommitted()) {
             return;
         }
-        if (characterEncoding == null) {
-            this.charset = null;
-            this.characterEncoding = null;
-            return;
-        }
 
-        this.characterEncoding = characterEncoding;
-        this.charset = B2CConverter.getCharset(characterEncoding);
-    }
-
-
-    public Charset getCharset() {
-        return charset;
+        charsetHolder = CharsetHolder.getInstance(characterEncoding);
+        charsetHolder.validate();
     }
 
 
     /**
-     * @return The name of the current encoding
+     * Returns the current character set.
+     *
+     * @return The current character set
+     *
+     * @deprecated Unused. Will be removed in Tomcat 12.
      */
+    @Deprecated
+    public Charset getCharset() {
+        return charsetHolder.getCharset();
+    }
+
+
+    /**
+     * Returns the name of the current encoding.
+     *
+     * @return The name of the current encoding
+     *
+     * @deprecated Unused. Will be removed in Tomcat 12.
+     */
+    @Deprecated
     public String getCharacterEncoding() {
-        return characterEncoding;
+        return charsetHolder.getName();
+    }
+
+
+    public CharsetHolder getCharsetHolder() {
+        return charsetHolder;
+    }
+
+
+    public void setCharsetHolder(CharsetHolder charsetHolder) {
+        this.charsetHolder = charsetHolder;
     }
 
 
@@ -567,8 +582,9 @@ public final class Response {
             this.contentType = m.toStringNoCharset();
             charsetValue = charsetValue.trim();
             if (!charsetValue.isEmpty()) {
+                charsetHolder = CharsetHolder.getInstance(charsetValue);
                 try {
-                    charset = B2CConverter.getCharset(charsetValue);
+                    charsetHolder.validate();
                 } catch (UnsupportedEncodingException e) {
                     log.warn(sm.getString("response.encoding.invalid", charsetValue), e);
                 }
@@ -583,9 +599,11 @@ public final class Response {
     public String getContentType() {
 
         String ret = contentType;
-
-        if (ret != null && charset != null) {
-            ret = ret + ";charset=" + characterEncoding;
+        if (ret != null) {
+            String charsetName = charsetHolder.getName();
+            if (charsetName != null) {
+                ret = ret + ";charset=" + charsetName;
+            }
         }
 
         return ret;
@@ -632,8 +650,7 @@ public final class Response {
         contentType = null;
         contentLanguage = null;
         locale = DEFAULT_LOCALE;
-        charset = null;
-        characterEncoding = null;
+        charsetHolder = CharsetHolder.EMPTY;
         contentLength = -1;
         status = 200;
         message = null;
@@ -751,10 +768,7 @@ public final class Response {
 
     public boolean isReady() {
         if (listener == null) {
-            if (log.isDebugEnabled()) {
-                log.debug(sm.getString("response.notNonBlocking"));
-            }
-            return false;
+            return true;
         }
         // Assume write is not possible
         boolean ready;
